@@ -204,7 +204,7 @@ print_version (FILE *stream, struct argp_state *state __attribute__ ((unused)))
 Copyright (C) %s Red Hat, Inc.\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
-"), "2007");
+"), "2008");
   fprintf (stream, gettext ("Written by %s.\n"), "Ulrich Drepper");
 }
 
@@ -383,6 +383,7 @@ handle_elf (int fd, Elf *elf, const char *prefix, const char *fname,
   Elf *debugelf = NULL;
   char *tmp_debug_fname = NULL;
   int result = 0;
+  size_t shdridx = 0;
   size_t shstrndx;
   struct shdr_info
   {
@@ -843,7 +844,7 @@ handle_elf (int fd, Elf *elf, const char *prefix, const char *fname,
 	  if (discard_section)
 	    debugshdr.sh_type = SHT_NOBITS;
 
-	  if (unlikely (gelf_update_shdr (scn, &debugshdr)) == 0)
+	  if (unlikely (gelf_update_shdr (scn, &debugshdr) == 0))
 	    /* There cannot be any overflows.  */
 	    INTERNAL_ERROR (fname);
 
@@ -881,7 +882,7 @@ handle_elf (int fd, Elf *elf, const char *prefix, const char *fname,
       debugehdr->e_flags = ehdr->e_flags;
       debugehdr->e_shstrndx = ehdr->e_shstrndx;
 
-      if (unlikely (gelf_update_ehdr (debugelf, debugehdr)) == 0)
+      if (unlikely (gelf_update_ehdr (debugelf, debugehdr) == 0))
 	{
 	  error (0, 0, gettext ("%s: error while creating ELF header: %s"),
 		 debug_fname, elf_errmsg (-1));
@@ -981,7 +982,7 @@ handle_elf (int fd, Elf *elf, const char *prefix, const char *fname,
     }
 
   /* Index of the section header table in the shdr_info array.  */
-  size_t shdridx = cnt;
+  shdridx = cnt;
 
   /* Add the section header string table section name.  */
   shdr_info[cnt].se = ebl_strtabadd (shst, ".shstrtab", 10);
@@ -1547,7 +1548,7 @@ handle_elf (int fd, Elf *elf, const char *prefix, const char *fname,
 	};
 
       /* Finally write the file.  */
-      if (unlikely (elf_update (debugelf, ELF_C_WRITE)) == -1)
+      if (unlikely (elf_update (debugelf, ELF_C_WRITE) == -1))
 	{
 	  error (0, 0, gettext ("while writing '%s': %s"),
 		 debug_fname, elf_errmsg (-1));
@@ -1772,3 +1773,6 @@ cannot set access and modification date of '%s'"), fname);
 
   return result;
 }
+
+
+#include "debugpred.h"
